@@ -55,6 +55,9 @@ History
       Oct 2021, Matthias Cuntz
     * v4.0, pyproject.toml, src layout, and Github actions,
       Oct 2021, Matthias Cuntz
+    * v4.1, final add_cyclic and has_cyclic as committed to cartopy,
+      Nov 2021, Matthias Cuntz
+    * v4.1.2, ncvue is gui_script entry_point, Jun 2022, Matthias Cuntz
 
 """
 # version, author
